// This is a constructor method that returns a new SpriterData
// object that is ready to be loaded with data from an SCML file
// using the load() method.
function SpriterData::create(%filename)
{
	// Create a new instance of a SpriterData class
	%this = new ScriptObject()
	{
		class = SpriterData;
	};

	if (%this)
	{
		%this._file = 0;
		%this._line = "";
		%this._xml = null;

		%this.scml_version = "0";
		%this.generator = "Spriter";
		%this.generator_version = "1.0";
		%this.folder_set = null;
		%this.entity_set = null;
		
		%this.entities_set = new SimSet();
	}

   if (%this.load(%filename))
      %this.setInternalName(%filename);
   else
   {
      %this.delete();
      %this = null;
   }

	return %this;
}

// This method is a destructor for the SpriterData object
function SpriterData::delete(%this)
{
	// Completely remove this object from memory
	if (%this.scml_version) %this.scml_version = "";
	if (%this.generator) %this.generator = "";
	if (%this.generator_version) %this.generator_version = "";
	if (%this.folder_set) %this.folder_set.delete();

	Parent::delete(%this);
}

// This method accepts a string as %filename and loads that file,
// populating all necessary SpriterData structures as it goes.
function SpriterData::load(%this, %filename)
{
	%result = false;
	%this._xml = new SimXMLDocument();
	%xml = %this._xml;

	if (%xml)
	{
		if (%xml.loadFile(%filename))
		{
			if (%xml.pushFirstChildElement("spriter_data"))
			{
				// Retrieve some of the metadata about the program that
				// created this SCML file and which version was targeted
				if (%xml.attributeExists("scml_version"))
					%this.scml_version = %xml.attributeF32("scml_version");
			
				if (%xml.attributeExists("generator"))
					%this.generator = %xml.attribute("generator");
			
				if (%xml.attributeExists("generator_version"))
					%this.generator_version = %xml.attribute("generator_version");

				%this.folder_set = %this.loadFolders();
				%this.entity_set = %this.loadEntities();
				
            if (%this.folder_set | %this.entity_set)
               %result = true;
			}
		}

		%xml.delete();
	}
	else
		echo("Unable to load '" @ %filename @ "'");

	return %result;
}

// SpriterData::loadFolders is a helper function for
// SpriterData::load() that reads the xml tags representing
// folders and their attributes.
function SpriterData::loadFolders(%this)
{
	%xml = %this._xml;

	// Attempt to read any folder data from the SCML file
	%folder = %xml.pushFirstChildElement("folder");

	if (%folder)
   	%folder_set = new SimSet();
   else
      %folder_set = null;

   if (%folder_set)
   {
      while (%folder)
      {
         %folderObject = SpriterFolder::create();

         if (%folderObject)
         {
            %attribute = %xml.firstAttribute();
            
            while (%attribute !$= "")
            {
               switch$(%attribute)
               {
                  case "id":
                     %folderObject.id = %xml.attributeS32(%attribute);
                     // echo(%attribute SPC "=" SPC %folderObject.id);
                  case "name":
                     %folderObject.name = %xml.attribute(%attribute);
                     // echo(%attribute SPC "=" SPC %folderObject.name);
                  default:
                     echo("Unknown attribute in folder element");
               }

               %attribute = %xml.nextAttribute();
            }
            
            %folderObject.file_set = %this.loadFiles(%folderObject.id);
            %folder_set.add(%folderObject);
         }
         
         %folder = %xml.nextSiblingElement("folder");
      }
   }
	
   %xml.popElement();	
	
	return %folder_set;
}

// SpriterData::loadFiles is a helper function for
// SpriterData::load() that reads the xml tags representing
// files and their attributes.
function SpriterData::loadFiles(%this, %folderId)
{
	%xml = %this._xml;   
   %set = null;

   %file = %xml.pushFirstChildElement("file");

   if (%file)
   {
      %set = new SimSet();  

      if (%set)
      {
         while (%file)
         {
           %fileObject = SpriterFile::create();
            
            if (%fileObject)
            {
               %attribute = %xml.firstAttribute();
               
               while (%attribute !$= "")
               {
                  switch$(%attribute)
                  {
                     case "type":
                        %fileObject.type = %xml.attribute(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.type);
                     case "id":
                        %fileObject.id = %xml.attributeS32(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.id);
                     case "name":
                        %fileObject.name = %xml.attribute(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.name);
                     case "width":
                        %fileObject.width = %xml.attributeS32(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.width);
                     case "height":
                        %fileObject.height = %xml.attributeS32(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.height);
                     case "pivot_x":
                        %fileObject.pivot_x = %xml.attributeF32(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.pivot_x);
                     case "pivot_y":
                        %fileObject.pivot_y = %xml.attributeF32(%attribute);
                        //echo(%attribute SPC "=" SPC %fileObject.pivot_y);
                     default:
                        echo("Unknown attribute in file element");
                  }

                  %attribute = %xml.nextAttribute();
               }

               %fileObject.name = "~/monster/" @ %fileObject.name;
               // Create an imagemap datablock for this file.
               if (isFile(%fileObject.name))
               {
                  %imageMap = new t2dImageMapDatablock()
                  {
                     imageName = %fileObject.name;
                     imageMode = full;

                     // Piggyback additional spriter specific data
                     type = %fileObject.type;
                     id = %fileObject.id;
                     name = %fileObject.name;
                     pivot = %fileObject.pivot_x SPC %fileObject.pivot_y;
                  };

                  %imageMap.size = %imageMap.getSrcBitmapSize();
                  %this.imageMap[%folderId, %fileObject.id] = %imageMap;
               }

               %set.add(%fileObject);
            }

            %file = %xml.nextSiblingElement("file");
         }
      }

      %xml.popElement();
   }
   
   return %set;
}

// SpriterData::loadEntities is a helper function for
// SpriterData::load() that reads the xml tags representing
// entities and their attributes.
function SpriterData::loadEntities(%this)
{
	%xml = %this._xml;
   %set = null;

   %entity = %xml.pushFirstChildElement("entity");

   if (%entity)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%entity)
         {
            %entityObject = SpriterEnt::create();      
            
            if (%entityObject)
            {
               %attribute = %xml.firstAttribute();
            
               while (%attribute !$= "")
               {
                  switch$(%attribute)
                  {
                     case "id":
                        %entityObject.id = %xml.attributeS32(%attribute);
                     //   echo(%attribute SPC "=" SPC %entityObject.id);
                     case "name":
                        %entityObject.name = %xml.attribute(%attribute);
                     //   echo(%attribute SPC "=" SPC %entityObject.name);
                     default:
                        echo("Unknown attribute in entity element");
                  }

                  %attribute = %xml.nextAttribute();
               }
               
               %this.entities_set.add(%entityObject);

               %entityObject.animation_set = %this.loadAnimations();
               %set.add(%entityObject);
            }

            %entity = %xml.nextSiblingElement("entity");
         }
      }

      %xml.popElement();
   }
   
   return %set;
}

// SpriterData::loadAnimations is a helper function for
// SpriterData::loadEntities() that reads the xml tags representing
// animations and their attributes.
function SpriterData::loadAnimations(%this)
{
	%xml = %this._xml;   
   %set = null;

   %anim = %xml.pushFirstChildElement("animation");

   if (%anim)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%anim)
         {
            %animObject = SpriterAnimation::create();      
            
            if (%animObject)
            {
               %attribute = %xml.firstAttribute();
               
               while (%attribute !$= "")
               {
                  switch$(%attribute)
                  {
                     case "id":
                        %animObject.id = %xml.attributeS32(%attribute);
                     //   echo("animation" SPC %attribute SPC "=" SPC %animObject.id);
                     case "name":
                        %animObject.name = %xml.attribute(%attribute);
                     //   echo(%attribute SPC "=" SPC %animObject.name);
                     case "length":
                        %animObject.length = %xml.attributeS32(%attribute);
                     //   echo(%attribute SPC "=" SPC %animObject.length);
                     case "looping":
                        %animObject.looping = %xml.attributeS32(%attribute);
                     //   echo(%attribute SPC "=" SPC %animObject.looping);
                     default:
                        echo("Unknown attribute in animation element");
                  }

                  %attribute = %xml.nextAttribute();
               }

               %animObject.mainline = %this.loadMainline();
               %animObject.timeline_set = %this.loadTimelines();

               %animObject.setInternalName(%animObject.name);
               %set.add(%animObject);
            }
            
            %anim = %xml.nextSiblingElement("animation");
         }
      }

      %xml.popElement();
   }
   
   return %set;
}

// SpriterData::loadMainline is a helper function for
// SpriterData::loadAnimations() that reads the xml tag representing
// the mainline and its attributes.
function SpriterData::loadMainline(%this)
{
	%xml = %this._xml;   
   %mainlineObject = null;

   %mainline = %xml.pushFirstChildElement("mainline");

   if (%mainline)
   {
      %mainlineObject = SpriterMainline::create();      
      
      if (%mainlineObject)
      {
         %mainlineObject.key_set = %this.loadKeys();
         echo(%mainlineObject.key_set.getCount() SPC "keys in mainline");
      }

      %xml.popElement();
   }
   
   return %mainlineObject;
}

// SpriterData::loadTimelines is a helper function for
// SpriterData::loadAnimations() that reads the xml tags representing
// timelines and their attributes.
function SpriterData::loadTimelines(%this)
{
	%xml = %this._xml;   
   %set = null;

   %timeline = %xml.pushFirstChildElement("timeline");

   if (%timeline)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%timeline)
         {
            %timelineObject = SpriterTimeline::create();      
            
            if (%timelineObject)
            {
               %attribute = %xml.firstAttribute();
               
               while (%attribute !$= "")
               {
                  switch$(%attribute)
                  {
                     case "id":
                        %timelineObject.id = %xml.attributeS32(%attribute);
                     //   echo("timeline" SPC %attribute SPC "=" SPC %timelineObject.id);
                     case "name":
                        %timelineObject.name = %xml.attribute(%attribute);
                     //   echo("timeline" SPC %attribute SPC "=" SPC %timelineObject.name);
                     default:
                        echo("Unknown attribute in timeline element");
                  }

                  %attribute = %xml.nextAttribute();
               }

               %timelineObject.key_set = %this.loadKeys();

               %set.add(%timelineObject);
            }

            %timeline = %xml.nextSiblingElement("timeline");
         }
      }

      %xml.popElement();
   }
   
   return %set;
}

// SpriterData::loadKeys is a helper function for
// SpriterData::loadMainline() and SpriterData::loadTimeline() that reads the
// xml tag representing the key frames and their attributes.
function SpriterData::loadKeys(%this)
{
	%xml = %this._xml;   
   %set = null;

   %key = %xml.pushFirstChildElement("key");

   if (%key)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%key)
         {
            %keyObject = SpriterKey::create();      
            
            if (%keyObject)
            {
               %attribute = %xml.firstAttribute();
               
               while (%attribute !$= "")
               {
                  switch$(%attribute)
                  {
                     case "id":
                        %keyObject.id = %xml.attributeS32(%attribute);
                     //   echo(%attribute SPC "=" SPC %keyObject.id);
                     case "time":
                        %keyObject.time = %xml.attribute(%attribute);
                     //   echo(%attribute SPC "=" SPC %keyObject.time);
                     case "curve_type":
                        %keyObject.curve_type = %xml.attribute(%attribute);
                     //   echo(%attribute SPC "=" SPC %keyObject.curve_type);
                     case "spin":
                        %keyObject.spin = %xml.attribute(%attribute);
                     //   echo(%attribute SPC "=" SPC %keyObject.spin);
                     default:
                        echo("Unknown attribute in key element");
                  }

                  %attribute = %xml.nextAttribute();
               }

               %keyObject.objects_set = %this.loadObjects();
               %set.add(%keyObject);
            }

            %key = %xml.nextSiblingElement("key");
         }

         %xml.popElement();
      }
   }
   
   return %set;
}

// SpriterData::loadObjects is a helper function for
// SpriterData::loadKeys() that reads the xml tags representing
// the objects and object references and their attributes.
function SpriterData::loadObjects(%this)
{
	%xml = %this._xml;   
   %set = null;
   %index = 0;

   %object = %xml.pushChildElement(%index);

   if (%object)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%object)
         {
            %objectObject = SpriterKey::create();      

            if (%objectObject)
            {
               //echo("text =" SPC %xml.elementValue());
               //echo("index =" SPC %index);

               if (%xml.elementValue() $= "object_ref")
               {
                  %objectObject.reference = true;

                  %attribute = %xml.firstAttribute();
                  
                  while (%attribute !$= "")
                  {
                     switch$(%attribute)
                     {
                        case "id":
                           %objectObject.id = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.id);
                        case "timeline":
                           %objectObject.timeline = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.timeline);
                        case "key":
                           %objectObject.key = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.key);
                        case "z_index":
                           %objectObject.z_index = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.z_index);
                        case "object_type":
                           %objectObject.object_type = %xml.attribute(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.object_type);
                        default:
                        //   echo("Unknown attribute in key element");
                     }

                     %set.add(%objectObject);

                     %attribute = %xml.nextAttribute();
                  } 
               }
               else if (%xml.elementValue() $= "object")
               {
                  %objectObject.reference = false;

                  %attribute = %xml.firstAttribute();
                  
                  while (%attribute !$= "")
                  {
                     switch$(%attribute)
                     {
                        case "folder":
                           %objectObject.folder = %xml.attributeS32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.folder);
                        case "file":
                           %objectObject.file = %xml.attributeS32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.file);
                        case "x":
                           %objectObject.x = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.x);
                        case "y":
                           %objectObject.y = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.y);
                        case "pivot_x":
                           %objectObject.pivot_x = %xml.attributeF32(%attribute);
                        //  echo(%attribute SPC "=" SPC %objectObject.pivot_x);
                        case "pivot_y":
                           %objectObject.pivot_y = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.pivot_y);
                        case "angle":
                           %objectObject.angle = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.angle);
                        case "scale_x":
                           %objectObject.scale_x = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.scale_x);
                        case "scale_y":
                           %objectObject.scale_y = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.scale_y);
                        case "a":
                           %objectObject.a = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.a);
                        default:
                        //   echo("Unknown attribute in key element");
                     }

                     %set.add(%objectObject);

                     %attribute = %xml.nextAttribute();
                  }
               }
            }

            %xml.popElement();
            %object = %xml.pushChildElement(%index++);
         }
      }
   }

   return %set;
}

/* - - - - - - - - - - - - - - Additional Classes - - - - - - - - - - - - - - */

// Class to represent spriter folders
function SpriterFolder::create()
{
	%this = new ScriptObject()
	{
		class = SpriterFolder;
	};

	if (%this)
	{
		%this.file_set = null;
		%this.id = 0;
		%this.name = "";
	}
	
	return %this;
}

function SpriterFolder::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter files
function SpriterFile::create()
{
	%this = new ScriptObject()
	{
		class = SpriterFile;
	};

	if (%this)
	{
		%this.type = "image";
		%this.id = 0;
		%this.name = "";
		%this.width = 0;
		%this.height = 0;
		%this.pivot_x = 0.0;
		%this.pivot_y = 1.0;
	}
	
	return %this;
}

function SpriterFile::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter entities
function SpriterEnt::create()
{
	%this = new ScriptObject()
	{
		class = SpriterEnt;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.name = "";
		%this.animation_set = null;
	}

	return %this;
}

function SpriterEnt::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter animations
function SpriterAnimation::create()
{
	%this = new ScriptObject()
	{
		class = SpriterAnimation;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.name = "";
		%this.length = 0;
		%this.looping = false;
		%this.mainline = null;
		%this.timeline_set = null;
	}

	return %this;
}

function SpriterAnimation::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter mainlines
function SpriterMainline::create()
{
	%this = new ScriptObject()
	{
		class = SpriterMainline;
	};
	
	if (%this)
		%this.key_set = null;

	return %this;
}

function SpriterMainline::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter timelines
function SpriterTimeline::create()
{
	%this = new ScriptObject()
	{
		class = SpriterTimeline;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.name = "";
		%this.object_type = "sprite";
		%this.variable_type = "string";
		%this.usage = "display";
		%this.key_set = null;
	}

	return %this;
}

function SpriterTimeline::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter animation key
function SpriterKey::create()
{
	%this = new ScriptObject()
	{
		class = SpriterKey;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.time = 0;
		%this.spin = 1;
		%this.object_set = null;
	}

	return %this;
}

function SpriterKey::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter objects
// and object references
function SpriterObject::create()
{
	%this = new ScriptObject()
	{
		class = SpriterObject;
	};
	
	if (%this)
	{
	   %this.reference = false;
	   
	   // Object reference fields
		%this.id = 0;
		%this.timeline = 0;
		%this.key = 0;
		%this.z_index = 0;
		%this.object_type = "sprite";
		
		// Object fields
		%this.folder = "";
		%this.file = "";
		%this.imageMap = null;
		%this.x = 0.0;
		%this.y = 0.0;
		%this.pivot_x = 0;
		%this.pivot_y = 0;
		%this.angle = 0.0;
		%this.scale_x = 1.0;
		%this.scale_y = 1.0;
		%this.a = 1.0;
	}

	return %this;
}

function SpriterObject::delete(%this)
{
	Parent::delete(%this);
}

/*----------------------------------------------------------------------------*/
