exec("./spriterEntity.cs");

// This is a constructor method that returns a new SpriterData
// object that is ready to be loaded with data from an SCML file
// using the load() method.
function SpriterData::create(%filename)
{
	// Create a new instance of a SpriterData class
	%this = new ScriptObject()
	{
		class = SpriterData;
	};

	if (%this)
	{
		%this._file = 0;
		%this._line = "";
		%this._xml = null;

		%this.scml_version = "0";
		%this.generator = "Spriter";
		%this.generator_version = "1.0";

		%this.entity_set = new SimSet();
	}

   if (%this.load(%filename))
      %this.setInternalName(%filename);
   else
   {
      %this.delete();
      %this = null;
   }

	return %this;
}

// This method is a destructor for the SpriterData object
function SpriterData::delete(%this)
{
	// Completely remove this object from memory
	if (%this.scml_version) %this.scml_version = "";
	if (%this.generator) %this.generator = "";
	if (%this.generator_version) %this.generator_version = "";
	if (%this.entity_set) %this.folder_set.delete();

	Parent::delete(%this);
}

// This method accepts a string as %filename and loads that file,
// populating all necessary SpriterData structures as it goes.
function SpriterData::load(%this, %filename)
{
	%result = false;
	%this._xml = new SimXMLDocument();
	%xml = %this._xml;

	if (%xml)
	{
		if (%xml.loadFile(%filename))
		{
			if (%xml.pushFirstChildElement("spriter_data"))
			{
				// Retrieve some of the metadata about the program that
				// created this SCML file and which version was targeted
				if (%xml.attributeExists("scml_version"))
					%this.scml_version = %xml.attributeF32("scml_version");
			
				if (%xml.attributeExists("generator"))
					%this.generator = %xml.attribute("generator");
			
				if (%xml.attributeExists("generator_version"))
					%this.generator_version = %xml.attribute("generator_version");

				%foldersLoaded = %this.loadFolders();
				%this.entity_set = %this.loadEntities();
				
            if (%foldersLoaded | %this.entity_set)
               %result = true;
			}
		}

		%xml.delete();
	}
	else
		echo("Unable to load '" @ %filename @ "'");

	return %result;
}

// SpriterData::loadFolders is a helper function for
// SpriterData::load() that reads the xml tags representing
// folders and their attributes.
function SpriterData::loadFolders(%this)
{
	%xml = %this._xml;

	// Attempt to read any folder data from the SCML file
	%folder = %xml.pushFirstChildElement("folder");

   while (%folder)
   {
      %attribute = %xml.firstAttribute();
            
      while (%attribute !$= "")
      {
         switch$(%attribute)
         {
            case "id":
               %folderId = %xml.attributeS32(%attribute);
            case "name":
               %folderName = %xml.attribute(%attribute);
            default:
               echo("Unknown attribute in folder element");
         }

         %attribute = %xml.nextAttribute();
      }
            
      %this.loadFiles(%folderId, %folderName);

      %folder = %xml.nextSiblingElement("folder");
   }
	
   %xml.popElement();

   return %true;
}

// SpriterData::loadFiles is a helper function for
// SpriterData::load() that reads the xml tags representing
// files and their attributes.
function SpriterData::loadFiles(%this, %folderId)
{
	%xml = %this._xml;   

   %file = %xml.pushFirstChildElement("file");

   if (%file)
   {
      while (%file)
      {
         %attribute = %xml.firstAttribute();
         
         while (%attribute !$= "")
         {
            switch$(%attribute)
            {
               case "type":
                  %fileObject.type = %xml.attribute(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.type);
               case "id":
                  %fileObject.id = %xml.attributeS32(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.id);
               case "name":
                  %fileObject.name = %xml.attribute(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.name);
               case "width":
                  %fileObject.width = %xml.attributeS32(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.width);
               case "height":
                  %fileObject.height = %xml.attributeS32(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.height);
               case "pivot_x":
                  %fileObject.pivot_x = %xml.attributeF32(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.pivot_x);
               case "pivot_y":
                  %fileObject.pivot_y = %xml.attributeF32(%attribute);
                  //echo(%attribute SPC "=" SPC %fileObject.pivot_y);
               default:
                  echo("Unknown attribute in file element");
            }

            %attribute = %xml.nextAttribute();
         }

         %fileObject.name = "~/monster/" @ %fileObject.name;
         // Create an imagemap datablock for this file.
         if (isFile(%fileObject.name))
         {
            %imageMap = new t2dImageMapDatablock()
            {
               imageName = %fileObject.name;
               imageMode = full;

               // Piggyback additional spriter specific data
               fileType = %fileObject.type;
               folderId = %folderId;
               folderName = %folderName;
               fileId = %fileObject.id;
               fileName = %fileObject.name;
               bitMapSize = %imageMap.getSrcBitmapSize();
               pivot = %fileObject.pivot_x SPC %fileObject.pivot_y;
            };

            %this.imageMap[%folderId, %fileObject.id] = %imageMap;
         }

         %file = %xml.nextSiblingElement("file");
      }


      %xml.popElement();
   }
   
   return true;
}

// SpriterData::loadEntities is a helper function for
// SpriterData::load() that reads the xml tags representing
// entities and their attributes.
function SpriterData::loadEntities(%this)
{
	%xml = %this._xml;

   %entity = %xml.pushFirstChildElement("entity");

   if (%entity)
   {
      while (%entity)
      {
         %entityObject = SpriterEntity::create();
         
         if (%entityObject)
         {
            %attribute = %xml.firstAttribute();
         
            while (%attribute !$= "")
            {
               switch$(%attribute)
               {
                  case "id":
                     %entityObject.id = %xml.attributeS32(%attribute);
                  case "name":
                     %entityObject.name = %xml.attribute(%attribute);
                  default:
                     echo("Unknown attribute in entity element");
               }

               %attribute = %xml.nextAttribute();
            }
            
            %this.entity_set.add(%entityObject);

            %entityObject.anim_set = %this.loadAnimations();
         }

         %entity = %xml.nextSiblingElement("entity");
      }

      %xml.popElement();
   }
   
   return true;
}

// SpriterData::loadAnimations is a helper function for
// SpriterData::loadEntities() that reads the xml tags representing
// animations and their attributes.
function SpriterData::loadAnimations(%this)
{
	%xml = %this._xml;   
   %set = null;

   %anim = %xml.pushFirstChildElement("animation");

   if (%anim)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%anim)
         {
            %attribute = %xml.firstAttribute();
               
            while (%attribute !$= "")
            {
               switch$(%attribute)
               {
                  case "id":
                     %id = %xml.attributeS32(%attribute);
                  case "name":
                     %name = %xml.attribute(%attribute);
                  case "length":
                     %length = %xml.attributeS32(%attribute);
                  case "looping":
                     %looping = %xml.attributeS32(%attribute);
                  default:
                     echo("Unknown attribute in animation element");
               }

               %attribute = %xml.nextAttribute();
            }

            %mainline = %this.loadMainline();
            %timeline_set = %this.loadTimelines();

            // Here we need to use the data collected in the mainline and
            // timelines to create an ordered list of frames, their sprites,
            // and the times, images, and positions for each sprite.
            //
            // First, check the mainline for the initial key.

            %anim = %id SPC %name SPC %length SPC %looping;

            if (%mainline)
            {
               %keys = %mainline.key_set.getCount();
               
               %anim = %anim SPC %keys;          
               
               // For each key in the mainline
               for (%i = 0; %i < %keys; %i++)
               {
                  %mKey = %mainline.key_set.getObject(%i);

                  // Find the key start time
                  %startTime = %mKey.time;

                  // Find the next key start time, if last key, use total anim time
                  if (%n == %keys)
                  {
                     %endTime = %length;
                  }
                  else
                  {
                     %endTime = %mainline.key_set.getObject(%i + 1).time;
                  }
                  // Key time = next key start time - key start time
                  %time = %endTime - %startTime;

                  %anim = %anim SPC %time;
                  
                  // Count number of objects (sprites) in key frame
                  %n = %mKey.objects_set.getCount();

                  %anim = %anim SPC %n;

                  for (%j = 0; %j < %n; %j++)
                  {
                     %obj = %mKey.objects_set.getObject(%j);
                     
                     //%obj.
                  }

                  echo(%anim);
               }

               %set.add(%anim);
            }
         }
            
         %anim = %xml.nextSiblingElement("animation");
      }

      %xml.popElement();
   }
   
   return %set;
}

// SpriterData::loadMainline is a helper function for
// SpriterData::loadAnimations() that reads the xml tag representing
// the mainline and its attributes.
function SpriterData::loadMainline(%this)
{
	%xml = %this._xml;   
   %mainlineObject = null;

   %mainline = %xml.pushFirstChildElement("mainline");

   if (%mainline)
   {
      %mainlineObject = SpriterMainline::create();      
      
      if (%mainlineObject)
      {
         %mainlineObject.key_set = %this.loadKeys();
         echo(%mainlineObject.key_set.getCount() SPC "keys in mainline");
      }

      %xml.popElement();
   }
   
   return %mainlineObject;
}

// SpriterData::loadTimelines is a helper function for
// SpriterData::loadAnimations() that reads the xml tags representing
// timelines and their attributes.
function SpriterData::loadTimelines(%this)
{
	%xml = %this._xml;   
   %set = null;

   %timeline = %xml.pushFirstChildElement("timeline");

   if (%timeline)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%timeline)
         {
            %timelineObject = SpriterTimeline::create();      
            
            if (%timelineObject)
            {
               %attribute = %xml.firstAttribute();
               
               while (%attribute !$= "")
               {
                  switch$(%attribute)
                  {
                     case "id":
                        %timelineObject.id = %xml.attributeS32(%attribute);
                     //   echo("timeline" SPC %attribute SPC "=" SPC %timelineObject.id);
                     case "name":
                        %timelineObject.name = %xml.attribute(%attribute);
                     //   echo("timeline" SPC %attribute SPC "=" SPC %timelineObject.name);
                     default:
                        echo("Unknown attribute in timeline element");
                  }

                  %attribute = %xml.nextAttribute();
               }

               %timelineObject.key_set = %this.loadKeys();

               %set.add(%timelineObject);
            }

            %timeline = %xml.nextSiblingElement("timeline");
         }
      }

      %xml.popElement();
   }
   
   return %set;
}

// SpriterData::loadKeys is a helper function for
// SpriterData::loadMainline() and SpriterData::loadTimeline() that reads the
// xml tag representing the key frames and their attributes.
function SpriterData::loadKeys(%this)
{
	%xml = %this._xml;   
   %set = null;

   %key = %xml.pushFirstChildElement("key");

   if (%key)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%key)
         {
            %attribute = %xml.firstAttribute();
               
            while (%attribute !$= "")
            {
               switch$(%attribute)
               {
                  case "id":
                     %id = %xml.attributeS32(%attribute);
                  case "time":
                     %time = %xml.attribute(%attribute);
                  case "curve_type":
                     %curve_type = %xml.attribute(%attribute);
                  case "spin":
                     %spin = %xml.attribute(%attribute);
                  default:
                     echo("Unknown attribute in key element");
               }

               %attribute = %xml.nextAttribute();
            }

            //%keyData = "" & %id SPC %time SPC %curve_type SPC %spin;

            %keyObject.objects_set = %this.loadObjects();
            %set.add(%keyObject);

            %key = %xml.nextSiblingElement("key");
         }

         %xml.popElement();
      }
   }
   
   return %set;
}

// SpriterData::loadObjects is a helper function for
// SpriterData::loadKeys() that reads the xml tags representing
// the objects and object references and their attributes.
function SpriterData::loadObjects(%this)
{
	%xml = %this._xml;   
   %set = null;
   %index = 0;

   %object = %xml.pushChildElement(%index);

   if (%object)
   {
      %set = new SimSet();

      if (%set)
      {
         while (%object)
         {
            %objectObject = SpriterKey::create();      

            if (%objectObject)
            {
               //echo("text =" SPC %xml.elementValue());
               //echo("index =" SPC %index);

               if (%xml.elementValue() $= "object_ref")
               {
                  %objectObject.reference = true;

                  %attribute = %xml.firstAttribute();
                  
                  while (%attribute !$= "")
                  {
                     switch$(%attribute)
                     {
                        case "id":
                           %objectObject.id = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.id);
                        case "timeline":
                           %objectObject.timeline = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.timeline);
                        case "key":
                           %objectObject.key = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.key);
                        case "z_index":
                           %objectObject.z_index = %xml.attributeS32(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.z_index);
                        case "object_type":
                           %objectObject.object_type = %xml.attribute(%attribute);
                           //echo(%attribute SPC "=" SPC %objectObject.object_type);
                        default:
                        //   echo("Unknown attribute in key element");
                     }

                     %set.add(%objectObject);

                     %attribute = %xml.nextAttribute();
                  } 
               }
               else if (%xml.elementValue() $= "object")
               {
                  %objectObject.reference = false;

                  %attribute = %xml.firstAttribute();
                  
                  while (%attribute !$= "")
                  {
                     switch$(%attribute)
                     {
                        case "folder":
                           %objectObject.folder = %xml.attributeS32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.folder);
                        case "file":
                           %objectObject.file = %xml.attributeS32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.file);
                        case "x":
                           %objectObject.x = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.x);
                        case "y":
                           %objectObject.y = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.y);
                        case "pivot_x":
                           %objectObject.pivot_x = %xml.attributeF32(%attribute);
                        //  echo(%attribute SPC "=" SPC %objectObject.pivot_x);
                        case "pivot_y":
                           %objectObject.pivot_y = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.pivot_y);
                        case "angle":
                           %objectObject.angle = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.angle);
                        case "scale_x":
                           %objectObject.scale_x = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.scale_x);
                        case "scale_y":
                           %objectObject.scale_y = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.scale_y);
                        case "a":
                           %objectObject.a = %xml.attributeF32(%attribute);
                        //   echo(%attribute SPC "=" SPC %objectObject.a);
                        default:
                        //   echo("Unknown attribute in key element");
                     }

                     %set.add(%objectObject);

                     %attribute = %xml.nextAttribute();
                  }
               }
            }

            %xml.popElement();
            %object = %xml.pushChildElement(%index++);
         }
      }
   }

   return %set;
}

/* - - - - - - - - - - - - - - Additional Classes - - - - - - - - - - - - - - */

// Class to represent spriter folders
function SpriterFolder::create()
{
	%this = new ScriptObject()
	{
		class = SpriterFolder;
	};

	if (%this)
	{
		%this.file_set = null;
		%this.id = 0;
		%this.name = "";
	}
	
	return %this;
}

function SpriterFolder::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter files
function SpriterFile::create()
{
	%this = new ScriptObject()
	{
		class = SpriterFile;
	};

	if (%this)
	{
		%this.type = "image";
		%this.id = 0;
		%this.name = "";
		%this.width = 0;
		%this.height = 0;
		%this.pivot_x = 0.0;
		%this.pivot_y = 1.0;
	}
	
	return %this;
}

function SpriterFile::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter entities
function SpriterEnt::create()
{
	%this = new ScriptObject()
	{
		class = SpriterEnt;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.name = "";
		%this.animation_set = null;
	}

	return %this;
}

function SpriterEnt::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter animations
function SpriterAnimation::create()
{
	%this = new ScriptObject()
	{
		class = SpriterAnimation;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.name = "";
		%this.length = 0;
		%this.looping = false;
		%this.mainline = null;
		%this.timeline_set = null;
		%this.active_sprite_set = null;
		%this.inactive_sprite_set = null;
	}

	return %this;
}

function SpriterAnimation::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter mainlines
function SpriterMainline::create()
{
	%this = new ScriptObject()
	{
		class = SpriterMainline;
	};
	
	if (%this)
		%this.key_set = null;

	return %this;
}

function SpriterMainline::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter timelines
function SpriterTimeline::create()
{
	%this = new ScriptObject()
	{
		class = SpriterTimeline;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.name = "";
		%this.object_type = "sprite";
		%this.variable_type = "string";
		%this.usage = "display";
		%this.key_set = null;
	}

	return %this;
}

function SpriterTimeline::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter animation key
function SpriterKey::create()
{
	%this = new ScriptObject()
	{
		class = SpriterKey;
	};
	
	if (%this)
	{
		%this.id = 0;
		%this.time = 0;
		%this.spin = 1;
		%this.object_set = null;
	}

	return %this;
}

function SpriterKey::delete(%this)
{
	Parent::delete(%this);
}

// Class to represent spriter objects
// and object references
function SpriterObject::create()
{
	%this = new ScriptObject()
	{
		class = SpriterObject;
	};
	
	if (%this)
	{
	   %this.reference = false;
	   
	   // Object reference fields
		%this.id = 0;
		%this.timeline = 0;
		%this.key = 0;
		%this.z_index = 0;
		%this.object_type = "sprite";
		
		// Object fields
		%this.folder = "";
		%this.file = "";
		%this.imageMap = null;
		%this.x = 0.0;
		%this.y = 0.0;
		%this.pivot_x = 0;
		%this.pivot_y = 0;
		%this.angle = 0.0;
		%this.scale_x = 1.0;
		%this.scale_y = 1.0;
		%this.a = 1.0;
	}

	return %this;
}

function SpriterObject::delete(%this)
{
	Parent::delete(%this);
}

/*----------------------------------------------------------------------------*/
